#include <stdio.h>
#include <string.h>
#include "slab.h"
#define GC_malloc malloc
#define GC_malloc_atomic malloc
#define GC_free free


Expression *
clc_create_tic_expression(void)
{
    Expression   *exp;
    exp = clc_alloc_expression(TIC_EXPRESSION);
    exp->u.tic_expression = NULL;
    return exp;
}

Expression *
clc_create_toc_expression(void)
{
    Expression   *exp;
    exp = clc_alloc_expression(TOC_EXPRESSION);
    exp->u.toc_expression = NULL;
    return exp;
}

Expression *
clc_create_max_expression(Expression *left,
				    Expression *right)
{
    Expression	*exp;

    exp = clc_alloc_expression(MAX_EXPRESSION);
    exp->u.max_expression.left = left;
    exp->u.max_expression.right = right;

    return exp;
}

Expression *
clc_create_con_expression(Expression *exp1,
				    Expression *next)
{
    Expression	*exp;

    exp = clc_alloc_expression(CON_EXPRESSION);
    exp->u.con_expression.exp = exp1;
    exp->u.con_expression.next = next;

    return exp;
}

Expression *
clc_create_size_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(SIZE_EXPRESSION);
    exp->u.size_expression  = operand;

    return exp;
}

Expression *
clc_create_split_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(SPLIT_EXPRESSION);
    exp->u.split_expression  = operand;

    return exp;
}

Expression *
clc_create_eval_expression(Expression *name,
				    Expression *argument)
{
    Expression	*exp;

    exp = clc_alloc_expression(EVAL_EXPRESSION);
    exp->u.eval_expression.name = name;
    exp->u.eval_expression.argument = argument;

    return exp;
}

Expression *
clc_create_name_expression(char *name)
{
    Expression	*exp;

    exp = clc_alloc_expression(NAME_EXPRESSION);
    exp->u.name_expression.name  = name;

    return exp;
}

Expression *
clc_create_dif_expression(Expression *operand, Expression *n, Expression *i)
{
    Expression	*exp;

    exp = clc_alloc_expression(DIF_EXPRESSION);
    exp->u.dif_expression.operand = operand;
    exp->u.dif_expression.n = n;
    exp->u.dif_expression.i = i;

    return exp;
}

Expression *
clc_create_interval_value_expression(Expression *lower, Expression *upper)
{
    Expression *exp;

    exp =clc_alloc_expression(INTERVAL_EXPRESSION);
    exp->u.interval_expression.lower =lower;
    exp->u.interval_expression.upper =upper;        
    return exp;
}

Expression *
clc_create_pow_expression(Expression *left, Expression *right)
{
    Expression *exp;

    exp =clc_alloc_expression(POW_EXPRESSION);
    exp->u.pow_expression.left = left;
    exp->u.pow_expression.right = right;        
    return exp;
}

int
clc_file_input(char *file_name)
{
    FILE *fp;
    char file[128] = "./example/";

    strcat(file, file_name); 
    fp = fopen(file, "r");
    if (fp == NULL) {
        printf("   Error:cannot open file!\n");
    } else {
        SLAB_interpret(clc_current_interpreter, fp, SLAB_FILE_INPUT_MODE);
        fclose(fp);
        SLAB_interpret(clc_current_interpreter, stdin, SLAB_TTY_INPUT_MODE);
    }
    return 0;
}

int
clc_help_init(void)
{
    FILE *fp;
    char str[512], *ptr;
    
    printf("Welcome to help.  Type topic to get help about the topic. To back to the usual mode, type $.\n
Topics are: \n");
    fp = popen("ls ./help", "r");

    while(1) {
        fgets(str, 512, fp);
        if (feof(fp)){
            break;
        }
        ptr = strchr(str, '\n');
        if (ptr!=NULL) {
            *ptr = '\0';
        }
        printf("%s\n",str);
    }
    pclose(fp);
    return 0;
}

void
clc_linpro_out(void)
{
    FILE *fp;
    char str[262144], *ptr;
    char help_file_name[128] = "./linpro/_out";

    if ((fp = fopen(help_file_name, "r")) == NULL) {
        fprintf(stderr, "linpro_out cannot be open\n");
        return 1;
    }

    while(1) {
        fgets(str, 261244, fp);
        if (feof(fp)){
            break;
        }
        ptr = strchr(str, '\n');
        if (ptr!=NULL) {
            *ptr = '\0';
        }
        printf("%s\n",str);
    }
    fclose(fp);
    return 0;
}

int
clc_help(char *help_identifier)
{
    FILE *fp;
    char str[262144], *ptr;
    char help_file_name[128] = "./help/";

    strcat(help_file_name, help_identifier); 
    if ((fp = fopen(help_file_name, "r")) == NULL) {
        fprintf(stderr, "help_topic cannot be found\n");
        return 1;
    }

    while(1) {
        fgets(str, 261244, fp);
        if (feof(fp)){
            break;
        }
        ptr = strchr(str, '\n');
        if (ptr!=NULL) {
            *ptr = '\0';
        }
        printf("%s\n",str);
    }
    fclose(fp);
    return 0;
}

void
clc_save_data(char *identifier, char *data_file)
{
    FILE *fp;
    Variable *pos;
    char file[128] = "./data/";

    strcat(file, data_file); 
    fp = fopen(file, "w");
    if (fp == NULL) {
        fprintf(stderr,"   Error:cannot open file!\n");
    } else {   
        for(pos = clc_current_interpreter->variable; pos; pos->next) {
            if(!strcmp(pos->name, identifier))
                break;
        }
        if (pos != NULL) {
            clc_fprint_value(pos->value, fp);
        }
    }
    fclose(fp);
}

void
clc_function_define(char *identifier, ParameterList *parameter_list,
		    Expression *expression_list)
{
    FunctionDefinition *f;

    if (f = clc_search_function(identifier)) 
    {
        f->parameter = parameter_list;
        f->expression_list = expression_list;
        f->storage = clc_current_interpreter->current_storage;
        clc_current_interpreter->current_storage = MEM_open_storage(0);
/*        f->next = clc_current_interpreter->function_list;
        clc_current_interpreter->function_list = f; */
/*
        clc_compile_error(FUNCTION_MULTIPLE_DEFINE_ERR, "(%s)\n", identifier);
	clc_reopen_current_storage();  
*/
	return;
    }
    f = clc_malloc(sizeof(FunctionDefinition));
    f->name = identifier;
    f->parameter = parameter_list;
    f->expression_list = expression_list;
    f->storage = clc_current_interpreter->current_storage;
    clc_current_interpreter->current_storage = MEM_open_storage(0);
    f->next = clc_current_interpreter->function_list;
    clc_current_interpreter->function_list = f;
}

ParameterList*
clc_create_parameter(char *identifier)
{
    ParameterList	*p;

    p = clc_malloc(sizeof(ParameterList));
    p->name = identifier;
    p->next = NULL;

    return p;
}

ParameterList *
chain_parameter(ParameterList *list, ParameterList *add)
{
    ParameterList *pos;

    for (pos = list; pos->next; pos = pos->next)
	;
    pos->next = add;

    return list;
}

ParameterList*
clc_chain_parameter(ParameterList *list,  char *identifier)
{
    ParameterList	*add;

    add = clc_create_parameter(identifier);

    return chain_parameter(list, add);
}

Expression *
clc_alloc_expression(ExpressionType type)
{
    Expression	*exp;

    exp = clc_malloc(sizeof(Expression));
    exp->type = type;

    return exp;
}

Expression *
clc_create_expression_list(Expression *expression)
{
    Expression	*exp;
    exp = clc_alloc_expression(EXPRESSION_LIST_EXPRESSION);
    exp->u.expression_list.expression = expression;
    exp->u.expression_list.next = NULL;
    return exp;
}

Expression *
clc_chain_expression_list(Expression *list, Expression *add)
{
    Expression	*exp;
    Expression	*pos;

    exp = clc_alloc_expression(EXPRESSION_LIST_EXPRESSION);
    exp->u.expression_list.expression = add;
    exp->u.expression_list.next = NULL;

    for (pos = list; pos->u.expression_list.next;
	 pos = pos->u.expression_list.next)
	;
    pos->u.expression_list.next = exp;

    return list;
}

Expression *
clc_create_assign_expression(char *variable,
			     Expression *operand)
{
    Expression *exp;
    exp = clc_alloc_expression(ASSIGN_EXPRESSION);
    exp->u.assign_expression.variable = variable;
    exp->u.assign_expression.operand = operand;
    return exp;
}

Expression *
clc_create_matrix_element_assign_expression2(char *variable,
   Expression *row, Expression *operand)
{
    Expression *exp;

    exp = clc_alloc_expression(MATRIX_ELEMENT_ASSIGN_EXPRESSION2);
    exp->u.matrix_element_assign_expression.variable = variable;
    exp->u.matrix_element_assign_expression.row = row;
    exp->u.matrix_element_assign_expression.operand = operand;
    return exp;
}

Expression *
clc_create_matrix_element_assign_expression(char *variable,
   Expression *row, Expression *column, Expression *operand)
{
    Expression *exp;

    exp = clc_alloc_expression(MATRIX_ELEMENT_ASSIGN_EXPRESSION);
    exp->u.matrix_element_assign_expression.variable = variable;
    exp->u.matrix_element_assign_expression.row = row;
    exp->u.matrix_element_assign_expression.column = column;
    exp->u.matrix_element_assign_expression.operand = operand;
    return exp;
}

Expression
convert_value_to_expression(Value *v)
{
    Expression	expr;

    if (v->type == INT_VALUE) {
	expr.type = INT_EXPRESSION;
	expr.u.int_value = v->u.int_value;
        return expr;
    } else if(v->type == DOUBLE_VALUE) {
        expr.type = DOUBLE_EXPRESSION;
        expr.u.double_value = v->u.double_value;
        return expr;
    } else if(v->type == MATRIX_VALUE) {
        expr.type = MATRIX_VALUE_EXPRESSION;
        expr.u.matrix_value.top = v->u.matrix_value.top;
        expr.u.matrix_value.row = v->u.matrix_value.row;
        expr.u.matrix_value.column = v->u.matrix_value.column;
        return expr;
    } else if(v->type == COMPLEX_VALUE) {
        expr.type = COMPLEX_EXPRESSION;
        expr.u.complex_value.r = v->u.complex_value.r;
        expr.u.complex_value.i = v->u.complex_value.i;
        return expr;
    } else if(v->type == INTERVAL_VALUE) {
        expr.type = INTERVAL_EXPRESSION;
        expr.u.interval_value.lower = v->u.interval_value.lower;
        expr.u.interval_value.upper = v->u.interval_value.upper;
        return expr;
    } else if(v->type == INTERVAL_MATRIX_VALUE_EXPRESSION) {
        expr.type = INTERVAL_MATRIX_VALUE_EXPRESSION;
        expr.u.interval_matrix_value.lower.top = v->u.interval_matrix_value.lower.top;
        expr.u.interval_matrix_value.lower.row = v->u.interval_matrix_value.lower.row;
        expr.u.interval_matrix_value.lower.column = v->u.interval_matrix_value.lower.column;
        expr.u.interval_matrix_value.upper.top = v->u.interval_matrix_value.upper.top;
        expr.u.interval_matrix_value.upper.row = v->u.interval_matrix_value.upper.row;
        expr.u.interval_matrix_value.upper.column = v->u.interval_matrix_value.upper.column;
        return expr;
    } else
    return expr;
}

Expression *
clc_create_binary_expression(ExpressionType operator,
			     Expression *left,
			     Expression *right)
{
    if ((left->type == INT_EXPRESSION
	 || left->type == DOUBLE_EXPRESSION)
	&& (right->type == INT_EXPRESSION
	    || right->type == DOUBLE_EXPRESSION)) {
	Value v;
	v = clc_eval_binary_expression(NULL, operator, left, right);
	/* Overwriting left hand expression. */
	*left = convert_value_to_expression(&v);

	return left;
    } else {
        if (left->type == MATRIX_EXPRESSION && right->type ==MATRIX_EXPRESSION) {
            Value v;
            v = clc_eval_binary_expression(NULL, operator, left, right);
            left->type = MATRIX_EXPRESSION;
            left->u.matrix_value = v.u.matrix_value;
            return left;
        } 
        else 
        {
	    Expression *exp;
	    exp = clc_alloc_expression(operator);
	    exp->u.binary_expression.left = left;
	    exp->u.binary_expression.right = right;
	    return exp;
        }
    }
}

Expression *
clc_create_up_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_up_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
	exp = clc_alloc_expression(UP_EXPRESSION);
	exp->u.up_expression = operand;
	return exp;
    }
}

Expression *
clc_create_down_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_down_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
	exp = clc_alloc_expression(DOWN_EXPRESSION);
	exp->u.down_expression = operand;
	return exp;
    }
}

Expression *
clc_create_down_expression2(void)
{
    Expression   *exp;
    exp = clc_alloc_expression(DOWN_EXPRESSION);
    exp->u.down_expression = NULL;
    return exp;
}


Expression *
clc_create_up_expression2(void)
{
    Expression   *exp;
    exp = clc_alloc_expression(UP_EXPRESSION);
    exp->u.up_expression = NULL;
    return exp;
}

Expression *
clc_create_near_expression2(void)
{
    Expression   *exp;
    exp = clc_alloc_expression(NEAR_EXPRESSION);
    exp->u.near_expression = NULL;
    return exp;
}

Expression *
clc_create_getbits_expression(Expression *r)
{
    Expression   *exp;
    exp = clc_alloc_expression(GETBITS_EXPRESSION);
    exp->u.getbits_expression = r;
    return exp;
}

Expression *
clc_create_complex_expression(double r,double i)
{
    Expression   *exp;
    exp = clc_alloc_expression(COMPLEX_EXPRESSION);
    exp->u.complex_value.r = r;
    exp->u.complex_value.i = i;
    return exp;
}

Expression *
clc_create_complex_expression1(double i)
{
    Expression   *exp;
    exp = clc_alloc_expression(COMPLEX_EXPRESSION);
    exp->u.complex_value.r = 0.;
    exp->u.complex_value.i = i;
    return exp;
}


Expression *
clc_create_near_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_near_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
	exp = clc_alloc_expression(NEAR_EXPRESSION);
	exp->u.near_expression = operand;
	return exp;
    }
}


Expression *
clc_create_inv_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_inv_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_inv_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(INV_EXPRESSION);
	    exp->u.inv_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_transpose_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_transpose_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_transpose_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(TRANSPOSE_EXPRESSION);
	    exp->u.transpose_expression = operand;
        } 
	return exp;
    }
}

Expression *
clc_create_veig_expression(Expression *operand)
{
    Expression	*exp;
    exp = clc_alloc_expression(VEIG_EXPRESSION);
    exp->u.veig_expression = operand;
    return exp;
}

Expression *
clc_create_tri_expression(Expression *operand)
{
    Expression	*exp;
    exp = clc_alloc_expression(TRI_EXPRESSION);
    exp->u.tri_expression = operand;
    return exp;
}

Expression *
clc_create_eig_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_eig_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_eig_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(EIG_EXPRESSION);
	    exp->u.eig_expression = operand;
        } 
	return exp;
    }
}

Expression *
clc_create_schur_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(SCHUR_EXPRESSION);
    exp->u.schur_expression = operand;
    return exp;
}

Expression *
clc_create_lu_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(LU_EXPRESSION);
    exp->u.lu_expression = operand;
    return exp;
}

Expression *
clc_create_upper_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(UPPER_EXPRESSION);
    exp->u.upper_expression = operand;
    return exp;
}

Expression *
clc_create_lower_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(LOWER_EXPRESSION);
    exp->u.lower_expression = operand;
    return exp;
}

Expression *
clc_create_real_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(REAL_EXPRESSION);
    exp->u.real_expression = operand;
    return exp;
}

Expression *
clc_create_imag_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(IMAG_EXPRESSION);
    exp->u.imag_expression = operand;
    return exp;
}

Expression *
clc_create_qr_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(QR_EXPRESSION);
    exp->u.qr_expression = operand;
    return exp;
}

Expression *
clc_create_chol_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(CHOL_EXPRESSION);
    exp->u.chol_expression = operand;
    return exp;
}

Expression *
clc_create_svd_expression(Expression *operand)
{
    Expression    *exp;
    exp = clc_alloc_expression(SVD_EXPRESSION);
    exp->u.svd_expression = operand;
    return exp;
}

Expression *
clc_create_vsin_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(VSIN_EXPRESSION);
    exp->u.vsin_expression = operand;

    return exp;
}

Expression *
clc_create_sin_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_sin_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_sin_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
/*             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row; */
        }
        else {
	    exp = clc_alloc_expression(SIN_EXPRESSION);
	    exp->u.sin_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_vcos_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(VCOS_EXPRESSION);
    exp->u.vcos_expression = operand;
       
    return exp;
}

Expression *
clc_create_cos_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_cos_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_cos_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(COS_EXPRESSION);
	    exp->u.cos_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_norm_expression(Expression *operand)
{
    Expression	*exp;
    Value   v;
    if (operand->type == DOUBLE_EXPRESSION ||
        operand->type == COMPLEX_EXPRESSION) {
	v = clc_eval_norm_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else if (operand->type == MATRIX_VALUE_EXPRESSION) { 
        v = clc_eval_norm_expression(NULL, operand);
        exp = clc_alloc_expression(DOUBLE_EXPRESSION);
        exp->u.double_value = v.u.double_value;
	return exp;
    } else {
	exp = clc_alloc_expression(NORM_EXPRESSION);
	exp->u.norm_expression = operand;
	return exp;
    } 
}

Expression *
clc_create_minus_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_minus_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_minus_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(MINUS_EXPRESSION);
	    exp->u.minus_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_vlog_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(VLOG_EXPRESSION);
    exp->u.vlog_expression = operand;

    return exp;
}

Expression *
clc_create_log_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_log_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_log_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(LOG_EXPRESSION);
	    exp->u.log_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_log10_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_log10_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_log10_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(LOGTEN_EXPRESSION);
	    exp->u.log10_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_vexp_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(VEXP_EXPRESSION);
    exp->u.vexp_expression = operand;
       
    return exp;
}

Expression *
clc_create_exp_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_exp_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_exp_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(EXP_EXPRESSION);
	    exp->u.exp_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_fabs_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_fabs_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_fabs_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        } else if (operand->type == INTERVAL_MATRIX_VALUE_EXPRESSION) {
             v = clc_eval_fabs_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        } else {
	    exp = clc_alloc_expression(FABS_EXPRESSION);
	    exp->u.fabs_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_tan_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_tan_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_tan_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(TAN_EXPRESSION);
	    exp->u.tan_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_vtan_expression(Expression *operand)
{
    Expression	*exp;

    exp = clc_alloc_expression(VTAN_EXPRESSION);
    exp->u.vtan_expression = operand;

    return exp;
}

Expression *
clc_create_atan_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_cos_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_atan_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(ATAN_EXPRESSION);
	    exp->u.atan_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_acos_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_cos_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_acos_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(ACOS_EXPRESSION);
	    exp->u.acos_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_asin_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_cos_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_asin_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(ASIN_EXPRESSION);
	    exp->u.asin_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_cosh_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_cosh_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_cosh_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(COSH_EXPRESSION);
	    exp->u.cosh_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_sinh_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_sinh_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_sinh_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(SINH_EXPRESSION);
	    exp->u.sinh_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_tanh_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_tanh_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_tanh_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(TANH_EXPRESSION);
	    exp->u.tanh_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_sqrt_expression(Expression *operand)
{
    if (operand->type == INT_EXPRESSION
	|| operand->type == DOUBLE_EXPRESSION) {
	Value	v;
	v = clc_eval_sqrt_expression(NULL, operand);
	/* Notice! Overwriting operand expression. */
	*operand = convert_value_to_expression(&v);
	return operand;
    } else {
	Expression	*exp;
        Value   v;
        int k, l;
        if (operand->type == MATRIX_VALUE_EXPRESSION) { 
             v = clc_eval_sqrt_expression(NULL, operand);
             exp = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             exp->u.matrix_value.top = v.u.matrix_value.top;
             exp->u.matrix_value.column = v.u.matrix_value.column;
             exp->u.matrix_value.row =v.u.matrix_value.row;
             k=v.u.matrix_value.column;
             l=v.u.matrix_value.row;
        }
        else {
	    exp = clc_alloc_expression(SQRT_EXPRESSION);
	    exp->u.sqrt_expression = operand;
        } 

	return exp;
    }
}

Expression *
clc_create_identifier_expression(char *identifier)
{
    Expression	*exp;

    exp = clc_alloc_expression(IDENTIFIER_EXPRESSION);
    exp->u.identifier = identifier;

    return exp;
}

Expression *
clc_create_couple_identifier_expression(char *identifier, char *name)
{
    Expression	*exp;

    exp = clc_alloc_expression(COUPLE_IDENTIFIER_EXPRESSION);
    exp->u.couple_identifier_expression.identifier = identifier;
    exp->u.couple_identifier_expression.name = name;

    return exp;
}

Expression *
clc_create_couple_expression(char *name_first, char *name_second)
{
    Expression	*exp;

    exp = clc_alloc_expression(COUPLE_EXPRESSION);
    exp->u.couple_expression.name_first  = name_first;
    exp->u.couple_expression.name_second = name_second;

    return exp;
}

Expression *
clc_create_triple_expression(char *name_first, char *name_second, char *name_third)
{
    Expression	*exp;
    exp = clc_alloc_expression(TRIPLE_EXPRESSION);
    exp->u.triple_expression.name_first  = name_first;
    exp->u.triple_expression.name_second = name_second;
    exp->u.triple_expression.name_third  = name_third;

    return exp;
}

Expression *
clc_create_print_expression(char *identifier)
{
    Expression	*exp;

    exp = clc_alloc_expression(PRINT_EXPRESSION);
    exp->u.identifier = identifier;

    return exp;
}

Expression *
clc_create_plot_expression(char *name_first, char *name_second)
{
    Expression	*exp;

    exp = clc_alloc_expression(PLOT_EXPRESSION);
    exp->u.plot_expression.name_first = name_first;
    exp->u.plot_expression.name_second = name_second;

    return exp;
}

Expression *
clc_create_plot2_expression(char *name_first, char *name_second)
{
    Expression	*exp;

    exp = clc_alloc_expression(PLOT2_EXPRESSION);
    exp->u.plot2_expression.name_first = name_first;
    exp->u.plot2_expression.name_second = name_second;

    return exp;
}

Expression *
clc_create_linpro_expression(char *name_first, char *name_second, char *name_third)
{
    Expression	*exp;

    exp = clc_alloc_expression(LINPRO_EXPRESSION);
    exp->u.linpro_expression.name_first = name_first;
    exp->u.linpro_expression.name_second = name_second;
    exp->u.linpro_expression.name_third = name_third;

    return exp;
}

Expression *
clc_create_plot3_expression(char *name_first, char *name_second, char *name_third)
{
    Expression	*exp;

    exp = clc_alloc_expression(PLOT3_EXPRESSION);
    exp->u.plot3_expression.name_first = name_first;
    exp->u.plot3_expression.name_second = name_second;
    exp->u.plot3_expression.name_third = name_third;

    return exp;
}

Expression *
clc_create_who_expression(void)
{
    Expression	*exp;

    exp = clc_alloc_expression(WHO_EXPRESSION);

    return exp;
}

Expression *
clc_create_for_expression(Expression *first_condition,
			  Expression *second_condition,
			  Expression *third_condition,
                          Expression *expression)
{
    Expression	*exp;

    exp = clc_alloc_expression(FOR_EXPRESSION);
    exp->u.for_expression.first_condition = first_condition;
    exp->u.for_expression.second_condition = second_condition;
    exp->u.for_expression.third_condition = third_condition;
    exp->u.for_expression.expression_list = expression;

    return exp;
}

Expression *
clc_create_if_expression(Expression *condition,
			 Expression *then_expression,
			 Expression *else_expression)
{
    Expression	*exp;

    exp = clc_alloc_expression(IF_EXPRESSION);
    exp->u.if_expression.condition = condition;
    exp->u.if_expression.then_expression = then_expression;
    exp->u.if_expression.else_expression = else_expression;

    return exp;
}

Expression *
clc_create_if_expression2(Expression *condition,
			 Expression *then_expression)
{
    Expression	*exp;

    exp = clc_alloc_expression(IF_EXPRESSION2);
    exp->u.if_expression2.condition = condition;
    exp->u.if_expression2.then_expression = then_expression;

    return exp;
}

Expression *
clc_create_while_expression(Expression *condition,
			    Expression *expression)
{
    Expression	*exp;

    exp = clc_alloc_expression(WHILE_EXPRESSION);
    exp->u.while_expression.condition = condition;
    exp->u.while_expression.expression_list = expression;

    return exp;
}

Expression *
clc_create_do_while_expression(Expression *expression,
			    Expression *condition)
{
    Expression	*exp;

    exp = clc_alloc_expression(DO_WHILE_EXPRESSION);
    exp->u.do_while_expression.expression_list = expression;
    exp->u.do_while_expression.condition = condition;

    return exp;
}

Expression *
clc_create_function_call_expression(char *func_name,
				    Expression *argument)
{
    Expression	*exp;

    exp = clc_alloc_expression(FUNCTION_CALL_EXPRESSION);
    exp->u.function_call_expression.identifier = func_name;
    exp->u.function_call_expression.argument = argument;

    return exp;
}

Expression *
clc_create_matrix_element_call_expression2(char *mat_name,
				    Expression *row)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_ELEMENT_CALL_EXPRESSION2);
    exp->u.matrix_element_call_expression.identifier = mat_name;
    exp->u.matrix_element_call_expression.row = row;
    return exp;
}

Expression *
clc_create_matrix_element_call_expression(char *mat_name,
				    Expression *row, Expression *column)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_ELEMENT_CALL_EXPRESSION);
    exp->u.matrix_element_call_expression.identifier = mat_name;
    exp->u.matrix_element_call_expression.row = row;
    exp->u.matrix_element_call_expression.column = column;
    return exp;
}

Expression *
clc_create_matrix_ones_expression(Expression *row, Expression *column)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_ONES_EXPRESSION);
    exp->u.matrix_ones_expression.row = row;
    exp->u.matrix_ones_expression.column = column;
    return exp;
}

Expression *
clc_create_matrix_eye_expression(Expression *row, Expression *column)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_EYE_EXPRESSION);
    exp->u.matrix_ones_expression.row = row;
    exp->u.matrix_ones_expression.column = column;
    return exp;
}

Expression *
clc_create_matrix_rand_expression(Expression *row, Expression *column)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_RAND_EXPRESSION);
    exp->u.matrix_rand_expression.row = row;
    exp->u.matrix_rand_expression.column = column;
    return exp;
}

Expression *
clc_create_fft_expression(Expression *data, Expression *number)
{
    Expression	*exp;

    exp = clc_alloc_expression(FFT_EXPRESSION);
    exp->u.fft_expression.data = data;
    exp->u.fft_expression.number = number;
    return exp;
}

Expression *
clc_create_ifft_expression(Expression *data, Expression *number)
{
    Expression	*exp;

    exp = clc_alloc_expression(IFFT_EXPRESSION);
    exp->u.ifft_expression.data = data;
    exp->u.ifft_expression.number = number;
    return exp;
}

Expression *
clc_create_lread_expression(void)
{
    Expression	*exp;

    exp = clc_alloc_expression(LREAD_EXPRESSION);
    return exp;
}

Expression *
clc_create_linspace_expression(Expression *start, Expression *end, 
                               Expression *number)
{
    Expression	*exp;

    exp = clc_alloc_expression(LINSPACE_EXPRESSION);
    exp->u.linspace_expression.start = start;
    exp->u.linspace_expression.end = end;
    exp->u.linspace_expression.number = number;
    return exp;
}

Expression *
clc_create_matrix_zeros_expression(Expression *row, Expression *column)
{
    Expression	*exp;

    exp = clc_alloc_expression(MATRIX_ZEROS_EXPRESSION);
    exp->u.matrix_zeros_expression.row = row;
    exp->u.matrix_zeros_expression.column = column;
    return exp;
}

/* Expression *
clc_is_matrix_expression(Expression *mat_el)
{
    Expression *mat;
    Expression *mat_el_t, *mat_el_s;
    long int n = 0, m, l=0;

    mat = clc_alloc_expression(MATRIX_EXPRESSION);

    if(mat_el == NULL) {
        return NULL;
    }
    else {
        mat_el_t = mat_el;
        l++;
        while(mat_el_t != NULL) {
            n++;
            mat_el_t = mat_el_t->u.matrix_element_expression.right;
        }
        mat_el_s = mat_el->u.matrix_element_expression.down;
        while(mat_el_s != NULL) {
            mat_el_t = mat_el_s;
            l++;
            m = 0;
            while(mat_el_t != NULL) {
                m++;
                mat_el_t = mat_el_t->u.matrix_element_expression.right;
            }
            if(n != m) return NULL;
                mat_el_s = mat_el_s->u.matrix_element_expression.down;
         }
         mat = clc_init_matrix_expression(mat_el, n, l);   
         return mat;
    }
} */

Expression *
clc_is_matrix_value_expression(LocalEnvironment *env, Expression *mat_el)
{
    Value v;
    Expression *mat;
    Expression *mat_el_t, *mat_el_s;
    long int n = 0, m, l=0;
    long int s, k;
    int matrix_type;
    int i;

    matrix_type = 1;


    if(mat_el == NULL) {
        return NULL;
    }
    else {
        mat_el_t = mat_el;

        v=eval_expression(env, mat_el_t);
        if( mat_el_t == NULL) 
        {
        } 
        else if( v.type == DOUBLE_VALUE) 
        {
        } 
        else if(v.type == COMPLEX_VALUE) 
        {
            if(matrix_type != 4) 
            {
                matrix_type = 2;
            }
        } 
        else if(v.type == DIF_VALUE) 
        {
            if(matrix_type != 4) 
            {
                matrix_type = 3;
                s = v.u.dif_value.n;
            }
        } 
        else 
        {
            matrix_type = 4;
        }
        l++;
        while(mat_el_t != NULL) {
            n++;
            mat_el_t = mat_el_t->u.matrix_element_expression.right;

            if( mat_el_t == NULL) 
            {
            } 
            else 
            {
                v=eval_expression(env, mat_el_t);
                if( v.type == DOUBLE_VALUE) 
                {
                } 
                else if(v.type == COMPLEX_VALUE) 
                {
                    if(matrix_type != 4) 
                    {
                        matrix_type = 2;
                    }
                } 
                else if(v.type == DIF_VALUE) 
                {
                    if(matrix_type != 4) 
                    {
                        matrix_type = 3;
                        s = v.u.dif_value.n;
                    }
                } 
                else 
                {
                    matrix_type = 4;
                }
            }  

        }
        mat_el_s = mat_el->u.matrix_element_expression.down;
        while(mat_el_s != NULL) 
        {
            mat_el_t = mat_el_s;

            if( mat_el_t == NULL) 
            {
            } 
            else 
            {
                v=eval_expression(env, mat_el_t);
                if( v.type == DOUBLE_VALUE) 
                {
                } 
                else if(v.type == COMPLEX_VALUE) 
                {
                    if(matrix_type != 4) 
                   {
                        matrix_type = 2;
                    }
                } 
                else if(v.type == DIF_VALUE) 
                {
                    if(matrix_type != 4) 
                    {
                        matrix_type = 3;
                        s = v.u.dif_value.n;
                    }
                } 
                else 
                {
                    matrix_type = 4;
                }
            }  

            l++;
            m = 0;
            while(mat_el_t != NULL) 
            {
                m++;
                mat_el_t = mat_el_t->u.matrix_element_expression.right;

                if( mat_el_t == NULL) 
                {
                } 
                else 
                {
                    v=eval_expression(env, mat_el_t);
                    if( v.type == DOUBLE_VALUE) 
                    {
                    } 
                    else if(v.type == COMPLEX_VALUE) 
                    {
                        if(matrix_type != 4) 
                        {
                            matrix_type = 2;
                        }
                    } 
                    else if(v.type == DIF_VALUE) 
                    {
                        if(matrix_type != 4) 
                        {
                            matrix_type = 3;
                            s = v.u.dif_value.n;
                        }
                    } 
                    else 
                    {
                        matrix_type = 4;
                    }
                }  
            }
            if(n != m) return NULL;
                mat_el_s = mat_el_s->u.matrix_element_expression.down;
         }
         if (matrix_type == 1) 
         {
             mat = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
             mat = clc_init_matrix_value_expression(env, mat_el, n, l, matrix_type);   
/*             v=clc_eval_matrix_value_expression(env, mat,n,l); */
         } 
         else if (matrix_type == 2) 
         {
             mat = clc_alloc_expression(COMPLEX_MATRIX_VALUE_EXPRESSION);
             mat = clc_init_matrix_value_expression(env, mat_el, n, l, matrix_type);   
 /*            v=clc_eval_complex_matrix_value_expression(env, mat,n,l); */
         } 
         else if (matrix_type == 3) 
         {
             mat = clc_alloc_expression(DIF_MATRIX_EXPRESSION);
             if((mat->u.difmatrix_value.value = 
                         (double *)GC_malloc_atomic(n*l * sizeof(double))) == NULL)
             {
                 fprintf(stderr, "memeory error\n");
                 return NULL;
             }
             for(i=0; i<n*l; i++) {
                 mat->u.difmatrix_value.value[i] = 0.;
             }
             if((mat->u.difmatrix_value.jacobi = 
                        (double *)GC_malloc_atomic(s*n*l * sizeof(double))) == NULL)
             {
                 fprintf(stderr, "memeory error\n");
                 return NULL;
             }
             for(i=0; i<s*n*l; i++) {
                 mat->u.difmatrix_value.jacobi[i] = 0.;
             }
             k = n*l;
             mat->u.difmatrix_value.n = k;
             n = 0;
             mat->u.difmatrix_value.column = s;
             mat_el_t = mat_el;
             while(mat_el_t != NULL) 
             {
                 mat_el_s = mat_el_t;
                 while(mat_el_s != NULL) 
                 {
                     v = clc_eval_matrix_element_expression(env,
                                  mat_el_s->u.matrix_element_expression.element); 
                     mat->u.difmatrix_value.value[n] = v.u.dif_value.value;
                     for(i=0; i<s; ++i)
                     {
                          mat->u.difmatrix_value.jacobi[k*i + n] 
                            = v.u.dif_value.jacobi[i];
                     } 
                     mat_el_s = mat_el_s->u.matrix_element_expression.down;
                     n++;
                 }
                 mat_el_t = mat_el_t->u.matrix_element_expression.right;
             }
         } 
         else 
         {
             fprintf(stderr, "this matrix type is not supported\n");
             return NULL; 
         }
         return mat;
    }
} 

Expression *
clc_init_matrix_expression(Expression *mat_el, long int column, long int row)
{
    Expression *mat;
    Expression *mat_el_s, *mat_el_t;
    int n = 0;

    mat = clc_make_matrix_expression(column, row);
    if(mat == NULL) return NULL;
    else {
        mat_el_t = mat_el;
        while(mat_el_t != NULL) {
            mat_el_s = mat_el_t;
            while(mat_el_s != NULL) {
                mat->u.matrix_expression.top[n] = 
                         *(mat_el_s->u.matrix_element_expression.element); 
                mat_el_s = mat_el_s->u.matrix_element_expression.down;
                n++;
            }
            mat_el_t = mat_el_t->u.matrix_element_expression.right;
        }
    }
    return mat;
}  

Expression *
clc_init_matrix_value_expression(LocalEnvironment *env, Expression *mat_el, 
                                 long int column, long int row, int matrix_type)
{
    Expression *mat;
    Expression *mat_el_s, *mat_el_t;
    Value v;
    int n = 0, i, j;
    mat = clc_make_matrix_value_expression(column, row, matrix_type);
    if(mat == NULL) return NULL;
    else {
        mat_el_t = mat_el;
        while(mat_el_t != NULL) {
            mat_el_s = mat_el_t;
            while(mat_el_s != NULL) {
                v = clc_eval_matrix_element_expression(env,
                                  mat_el_s->u.matrix_element_expression.element); 
                if (matrix_type == 1) 
                { 
                    mat->u.matrix_value.top[n] = v.u.double_value; 
                } 
                else if (matrix_type ==2) 
                {
                    if (v.type == DOUBLE_VALUE) 
                    {
                        mat->u.complex_matrix_value.top[n].r = v.u.double_value;
                        mat->u.complex_matrix_value.top[n].i = 0.0;
                    } 
                    else 
                    {
                        mat->u.complex_matrix_value.top[n].r = 
                                                v.u.complex_value.r;
                        mat->u.complex_matrix_value.top[n].i = 
                                                v.u.complex_value.i;
                    }
                }  
                else 
                {
                    fprintf(stderr, "Sorry, but this type of matrix is not allowed\n");
                }
                mat_el_s = mat_el_s->u.matrix_element_expression.down;
                n++;
            }
            mat_el_t = mat_el_t->u.matrix_element_expression.right;
        }
    }
    return mat;
}  

Expression *
clc_make_matrix_expression(long int column, long int row) 
{
    Expression *mat;

    mat = clc_alloc_expression(MATRIX_EXPRESSION);
    
    if((mat->u.matrix_expression.top = 
                (Expression *)GC_malloc(column*row * sizeof(Expression))) == NULL)
    {
        fprintf(stderr,"memeory error\n");
        return NULL;
    }

    mat->u.matrix_expression.column = column;
    mat->u.matrix_expression.row = row;

    return mat;
} 
     
Expression *
clc_make_matrix_value_expression(long int column, long int row, int matrix_type) 
{
    Expression *mat;
    int i;

    if (matrix_type == 1) {
        mat = clc_alloc_expression(MATRIX_VALUE_EXPRESSION);
        if((mat->u.matrix_value.top = 
                    (double *)GC_malloc_atomic(column*row * sizeof(double))) == NULL)
        {
            fprintf(stderr, "memeory error\n");
            return NULL;
        }
        for(i=0; i<column*row; i++) {
            mat->u.matrix_value.top[i] = 0.;
        }
        mat->u.matrix_value.column = column;
        mat->u.matrix_value.row = row;
    } else if (matrix_type == 2) {
        mat = clc_alloc_expression(COMPLEX_MATRIX_VALUE_EXPRESSION);
        if((mat->u.complex_matrix_value.top = 
                    (Complex *)GC_malloc(column*row * sizeof(Complex))) == NULL)
        {
            fprintf(stderr, "memeory error\n");
            return NULL;
        }
        for(i=0; i<column*row; i++) {
            mat->u.complex_matrix_value.top[i].r = 0.;
            mat->u.complex_matrix_value.top[i].i = 0.;
        }
        mat->u.complex_matrix_value.column = column;
        mat->u.complex_matrix_value.row = row;
    } else {
        fprintf(stderr, "this matrix type is not treated\n");
        return NULL;
    }
    return mat;
} 
     
Expression *
clc_create_matrix(Expression *matrix_element)
{
    Expression   *exp;
    exp = clc_alloc_expression(MATRIX);
    
    exp->u.matrix.matrix_element = matrix_element;

    return exp; 

} 

Expression *
clc_create_matrix_element_expression(Expression *element)
{
    Expression   *exp;
    exp = clc_alloc_expression(MATRIX_ELEMENT_EXPRESSION);
    
    exp->u.matrix_element_expression.element = element;
    exp->u.matrix_element_expression.right = NULL;
    exp->u.matrix_element_expression.down = NULL; 

    return exp; 

} 


Expression *
clc_column_add_matrix(Expression *el, Expression *root){
  Expression *column, *t, *s=NULL;

  column=clc_create_matrix_element_expression(el);

  if(column!=NULL){
    t=root;
    while(t->u.matrix_element_expression.down!=NULL){
      s=t;
      t=t->u.matrix_element_expression.down;
    }
    while(t->u.matrix_element_expression.right!=NULL){
      t=t->u.matrix_element_expression.right;
      if(s!=NULL){
        if(s->u.matrix_element_expression.right!=NULL){
          s=s->u.matrix_element_expression.right;
        }
        else return NULL;
      }
    }
    t->u.matrix_element_expression.right=column;
    if(s!=NULL){
      if(s->u.matrix_element_expression.right!=NULL){
        s=s->u.matrix_element_expression.right;
        s->u.matrix_element_expression.down=column;
      }
      else return NULL;
    }
    return root;
  }
  else return NULL;
}

Expression *
clc_row_add_matrix(Expression *el, Expression *root){
  Expression *row, *t;

  row=clc_create_matrix_element_expression(el);

  if(row != NULL){
   t=root;
   while(t->u.matrix_element_expression.down!=NULL){
     t=t->u.matrix_element_expression.down;
   }
   t->u.matrix_element_expression.down=row;
   return root;
  }
  else return NULL;
}
