#ifndef PRIVATE_CALC_H_INCLUDED
#define PRIVATE_CALC_H_INCLUDED
#include <stdio.h>
#include <setjmp.h>
#include "SLAB_MEMORY.h"

#define GC_malloc malloc
#define GC_free free
#define smaller(a, b) ((a) < (b) ? (a) : (b))
#define larger(a, b) ((a) > (b) ? (a) : (b))
typedef enum {
    PARSE_ERR = 1,
    CHARACTER_INVALID_ERR,
    FUNCTION_MULTIPLE_DEFINE_ERR,
    COMPILE_ERROR_NUM
} CompileError;

typedef enum {
    VARIABLE_NOT_FOUND_ERR = 1,
    FUNCTION_NOT_FOUND_ERR,
    BOOLEAN_EXPECTED_ERR,
    ARGUMENT_TOO_MANY_ERR,
    ARGUMENT_TOO_FEW_ERR,
    DOUBLE_EXPECTED_ERR,
    ARGUMENT_OUT_OF_RANGE_ERR,
    NOT_SQUARE_MATRIX_ERR,
    NOT_SYMMETRIC_MATRIX_ERR,
    NOT_MATRIX_ERR,
    NOT_SAME_DIMENSION_MATRIX_ERR,
    CANNOT_OPEN_FILE_ERR,
    UNDEFINED_INTERVAL_ERR,
    RUNTIME_ERROR_NUM
} RuntimeError;

typedef struct Expression_tag Expression;

typedef enum {
    INT_EXPRESSION = 1,
    DOUBLE_EXPRESSION,
    COMPLEX_EXPRESSION,
    MATRIX,
    MATRIX_EXPRESSION,
    DIF_EXPRESSION,
    DIF_MATRIX_EXPRESSION,
    SPLIT_EXPRESSION,
    SIZE_EXPRESSION,
    CON_EXPRESSION,
    NAME_EXPRESSION,
    EVAL_EXPRESSION,
    MATRIX_VALUE_EXPRESSION,
    COMPLEX_MATRIX_VALUE_EXPRESSION,
    MATRIX_ELEMENT_EXPRESSION,
    MATRIX_ELEMENT_CALL_EXPRESSION,
    MATRIX_ELEMENT_CALL_EXPRESSION2,
    MATRIX_ELEMENT_ASSIGN_EXPRESSION,
    MATRIX_ELEMENT_ASSIGN_EXPRESSION2,
    MATRIX_ONES_EXPRESSION,
    MATRIX_ZEROS_EXPRESSION,
    MATRIX_EYE_EXPRESSION,
    MATRIX_RAND_EXPRESSION,
    FFT_EXPRESSION,
    IFFT_EXPRESSION,
    LREAD_EXPRESSION,
    LINSPACE_EXPRESSION,
    INTERVAL_EXPRESSION,
    INTERVAL_VALUE_EXPRESSION,
    INTERVAL_MATRIX_EXPRESSION,
    INTERVAL_MATRIX_VALUE_EXPRESSION,
    IDENTIFIER_EXPRESSION,
    COUPLE_EXPRESSION,
    TRIPLE_EXPRESSION,
    COUPLE_IDENTIFIER_EXPRESSION,
    PRINT_EXPRESSION,
    LINPRO_EXPRESSION,
    PLOT_EXPRESSION,
    PLOT2_EXPRESSION,
    PLOT3_EXPRESSION,
    WHO_EXPRESSION,
    SAVE_EXPRESSION,
    EXPRESSION_LIST_EXPRESSION,
    ASSIGN_EXPRESSION,
    ADD_EXPRESSION,
    SUB_EXPRESSION,
    MUL_EXPRESSION,
    DIV_EXPRESSION,
    SOLVE_EXPRESSION,
    EXACT_SOLVE_EXPRESSION,
    MOD_EXPRESSION,
    EQ_EXPRESSION,
    NE_EXPRESSION,
    OR_EXPRESSION,
    AND_EXPRESSION,
    GT_EXPRESSION,
    GE_EXPRESSION,
    LT_EXPRESSION,
    LE_EXPRESSION,
    MINUS_EXPRESSION,
    NORM_EXPRESSION,
    INV_EXPRESSION,
    TIC_EXPRESSION,
    TOC_EXPRESSION,
    REAL_EXPRESSION,
    IMAG_EXPRESSION,
    UPPER_EXPRESSION,
    LOWER_EXPRESSION,
    MAX_EXPRESSION,
    TRANSPOSE_EXPRESSION,
    EIG_EXPRESSION,
    TRI_EXPRESSION,
    VEIG_EXPRESSION,
    SCHUR_EXPRESSION,
    LU_EXPRESSION,
    QR_EXPRESSION,
    CHOL_EXPRESSION,
    SVD_EXPRESSION,
    SIN_EXPRESSION,
    VSIN_EXPRESSION,
    POW_EXPRESSION,
    COS_EXPRESSION,
    VCOS_EXPRESSION,
    LOG_EXPRESSION,
    VLOG_EXPRESSION,
    LOGTEN_EXPRESSION,
    EXP_EXPRESSION,
    VEXP_EXPRESSION,
    FABS_EXPRESSION,
    TAN_EXPRESSION,
    VTAN_EXPRESSION,
    ATAN_EXPRESSION,
    ACOS_EXPRESSION,
    COSH_EXPRESSION,
    SINH_EXPRESSION,
    TANH_EXPRESSION,
    SQRT_EXPRESSION,
    ASIN_EXPRESSION,
    UP_EXPRESSION,
    DOWN_EXPRESSION,
    NEAR_EXPRESSION,
    GETBITS_EXPRESSION,
    FOR_EXPRESSION,
    IF_EXPRESSION,
    IF_EXPRESSION2,
    WHILE_EXPRESSION,
    DO_WHILE_EXPRESSION,
    FUNCTION_CALL_EXPRESSION,
    EXPRESSION_TYPE_NUM,
} ExpressionType;

typedef struct {
    Expression *expression;
    Expression *next;
} ExpressionList;

typedef struct {
    Expression *exp;
    Expression *next;
} ConExpression;

typedef struct {
    Expression *left;
    Expression *right;
} MaxExpression;

typedef struct {
    char	*variable;
    Expression	*operand;
} AssignExpression;

typedef struct {
    char *name_first;
    char *name_second;
    char *name_third;
} LinproExpression;

typedef struct {
    char *name_first;
    char *name_second;
} Plot2Expression;

typedef struct {
    char *name_first;
    char *name_second;
    char *name_third;
} Plot3Expression;

typedef struct {
    char	*name_first;
    char	*name_second;
} CoupleExpression;

typedef struct {
    char	*name_first;
    char	*name_second;
    char	*name_third;
} TripleExpression;

typedef struct {
    char	*name;
} NameExpression;

typedef struct {
    char	*identifier;
    char	*name;
} CoupleIdentifierExpression;

typedef struct {
    Expression	*left;
    Expression	*right;
} BinaryExpression;

typedef struct {
    Expression	*left;
    Expression	*right;
} PowExpression;

typedef struct {
    Expression	*first_condition;
    Expression	*second_condition;
    Expression	*third_condition;
    Expression	*expression_list;
} ForExpression;

typedef struct {
    Expression	*condition;
    Expression	*then_expression;
    Expression	*else_expression;
} IfExpression;

typedef struct {
    Expression	*condition;
    Expression	*then_expression;
} IfExpression2;

typedef struct {
    Expression	*condition;
    Expression	*expression_list;
} WhileExpression;

typedef struct {
    Expression	*expression_list;
    Expression	*condition;
} DoWhileExpression;

typedef struct {
    Expression 	*name;
    Expression	*argument;
} EvalExpression;

typedef struct {
    char	*identifier;
    Expression	*argument;
} FunctionCallExpression;

typedef struct {
    char        *identifier;
    char        *data_file;
} SaveExpression;

typedef struct {
    char        *variable;
    Expression  *row;
    Expression  *column;
    Expression  *operand;
} MatrixElementAssignExpression;

typedef struct {
    char        *variable;
    Expression  *row;
    Expression  *operand;
} MatrixElementAssignExpression2;

typedef struct {
    char        *identifier;
    Expression  *row;
    Expression  *column;
} MatrixElementCallExpression;

typedef struct {
    char        *identifier;
    Expression  *row;
} MatrixElementCallExpression2;

typedef struct {
    Expression  *row;
    Expression  *column;
} MatrixOnesExpression;

typedef struct {
    Expression  *row;
    Expression  *column;
} MatrixZerosExpression;

typedef struct {
    Expression  *row;
    Expression  *column;
} MatrixEyeExpression;

typedef struct {
    Expression  *row;
    Expression  *column;
} MatrixRandExpression;

typedef struct {
    Expression  *data;
    Expression  *number;
} FftExpression;

typedef struct {
    Expression  *data;
    Expression  *number;
} IfftExpression;

typedef struct {
    Expression  *start;
    Expression  *end;
    Expression  *number;
} LinspaceExpression;

typedef struct {
    Expression *top;
    long int column, row;
} MatrixExpression;

typedef struct {
    Expression *operand;
    Expression *n;
    Expression *i;
} DifExpression;

typedef struct {
    double *top;
    long int column, row;
} matr;

typedef struct {
    double value;
    double *jacobi;
    long int n;
} dif;

typedef struct {
    double *value;
    double *jacobi;
    long int n,column;
} difmatrix;

typedef struct {
    char *name;
} name;

typedef struct {
    double r;
    double i;
} Complex;

typedef struct {
    Complex *top;
    long int column, row;
} Complexmatr;

typedef struct {
    Expression *r;
    Expression *i;
} ComplexExpression;

typedef struct {
    double lower;
    double upper;
} Interval;

typedef struct {
    Expression *lower;
    Expression *upper;
} IntervalExpression;

typedef struct {
    matr lower;
    matr upper;
} Intervalmatr;

typedef struct {
    Expression *lower;
    Expression *upper;
} IntervalMatrixExpression;

typedef struct {
    Expression *element;
    Expression *right;
    Expression *down; 
} MatrixElementExpression;

typedef struct {
    Expression *matrix_element;
} Matrix;

struct Expression_tag {
    ExpressionType type;
    union {
	int			int_value;
	double			double_value;
        matr                    matrix_value;
        Complex                 complex_value;
        Complexmatr             complex_matrix_value;
        Interval                interval_value;
        Intervalmatr            interval_matrix_value; 
        IntervalMatrixExpression  interval_matrix_expression; 
	char			*identifier;
        CoupleExpression        couple_expression;
        TripleExpression        triple_expression;
        CoupleIdentifierExpression        couple_identifier_expression;
	ExpressionList		expression_list;
	AssignExpression	assign_expression;
	MatrixElementAssignExpression	matrix_element_assign_expression;
	MatrixElementAssignExpression2	matrix_element_assign_expression2;
	BinaryExpression	binary_expression;
	DifExpression		dif_expression;
	ConExpression		con_expression;
	MaxExpression		max_expression;
        difmatrix               difmatrix_value;
	NameExpression		name_expression;
	EvalExpression		eval_expression;
	PowExpression	        pow_expression;
	Expression		*minus_expression;
	Expression		*norm_expression;
	Expression		*inv_expression;
	Expression		*tic_expression;
	Expression		*toc_expression;
	Expression		*real_expression;
	Expression		*imag_expression;
	Expression		*upper_expression;
	Expression		*lower_expression;
	Expression		*transpose_expression;
	Expression		*eig_expression;
	Expression		*tri_expression;
	Expression		*veig_expression;
	Expression		*schur_expression;
	Expression		*lu_expression;
	Expression		*qr_expression;
	Expression		*chol_expression;
	Expression		*svd_expression;
	Expression		*vsin_expression;
	Expression		*sin_expression;
	Expression		*cos_expression;
	Expression		*vcos_expression;
	Expression		*log_expression;
	Expression		*vlog_expression;
	Expression		*log10_expression;
	Expression		*exp_expression;
	Expression		*vexp_expression;
	Expression		*fabs_expression;
	Expression		*tan_expression;
	Expression		*vtan_expression;
	Expression		*atan_expression;
	Expression		*acos_expression;
	Expression		*asin_expression;
	Expression		*cosh_expression;
	Expression		*sinh_expression;
	Expression		*tanh_expression;
	Expression		*sqrt_expression;
	Expression		*up_expression;
	Expression		*down_expression;
	Expression		*near_expression;
	Expression		*getbits_expression;
        Expression              *lread_expression;
        Expression              *split_expression;
        Expression              *size_expression;
	ForExpression		for_expression;
	IfExpression		if_expression;
	IfExpression2		if_expression2;
	FunctionCallExpression	function_call_expression;
	WhileExpression		while_expression;
	DoWhileExpression	do_while_expression;
        Matrix                  matrix;
        MatrixExpression        matrix_expression;
        ComplexExpression       complex_expression;
        IntervalExpression      interval_expression;
        MatrixOnesExpression    matrix_ones_expression;
        MatrixZerosExpression   matrix_zeros_expression;
        MatrixEyeExpression     matrix_eye_expression;
        MatrixRandExpression    matrix_rand_expression;
        FftExpression           fft_expression;
        IfftExpression          ifft_expression;
        LinspaceExpression      linspace_expression;
        MatrixElementExpression      matrix_element_expression;
        MatrixElementCallExpression  matrix_element_call_expression;
        MatrixElementCallExpression2  matrix_element_call_expression2;
        SaveExpression          save_expression;
        Plot2Expression         plot_expression;
        Plot2Expression         plot2_expression;
        Plot3Expression         plot3_expression;
        LinproExpression        linpro_expression;
    } u;
};

typedef struct ParameterList_tag {
    char	*name;
    struct ParameterList_tag *next;
} ParameterList;

typedef struct FunctionDefinition_tag {
    char		*name;
    ParameterList	*parameter;
    Expression		*expression_list;
    MEM_Storage		storage;
    struct FunctionDefinition_tag	*next;
} FunctionDefinition;

typedef enum {
    INT_VALUE = 1,
    DOUBLE_VALUE,
    COMPLEX_VALUE,
    INTERVAL_VALUE,
    INTERVAL_MATRIX_VALUE,
    MATRIX_VALUE,
    COMPLEX_MATRIX_VALUE,
    COUPLE_VALUE,
    TRIPLE_VALUE,
    DIF_VALUE,
    DIF_MATRIX_VALUE,
    NAME_VALUE,
    NULL_VALUE
} ValueType;

typedef struct {
    char *name_first;
    ValueType  type_first;
    union {
	int	int_value;
	double	double_value;
        Complex complex_value;
        Interval interval_value;
        Intervalmatr interval_matrix_value;
        matr    matrix_value;
        Complexmatr complex_matrix_value;
        dif     dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
    } u_first;
    char *name_second;
    ValueType  type_second;
    union {
	int	int_value;
	double	double_value;
        Complex complex_value;
        Interval interval_value;
        Intervalmatr interval_matrix_value;
        matr    matrix_value;
        Complexmatr complex_matrix_value;
        dif     dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
    } u_second;
} Couple;

typedef struct {
    char *name_first;
    ValueType  type_first;
    union {
	int	int_value;
	double	double_value;
        Complex complex_value;
        Interval interval_value;
        Intervalmatr interval_matrix_value;
        matr    matrix_value;
        Complexmatr complex_matrix_value;
        dif     dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
    } u_first;
    char *name_second;
    ValueType  type_second;
    union {
	int	int_value;
	double	double_value;
        Complex complex_value;
        Interval interval_value;
        Intervalmatr interval_matrix_value;
        matr    matrix_value;
        Complexmatr complex_matrix_value;
        dif     dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
    } u_second;
    char *name_third;
    ValueType  type_third;
    union {
	int	int_value;
	double	double_value;
        Complex complex_value;
        Interval interval_value;
        Intervalmatr interval_matrix_value;
        matr    matrix_value;
        Complexmatr complex_matrix_value;
        dif     dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
    } u_third;
} Triple;

typedef struct {
    ValueType	type;
    union {
	int	     int_value;
	double	     double_value;
        Complex      complex_value;
        Interval     interval_value;
        Intervalmatr interval_matrix_value;
        matr         matrix_value;
        Complexmatr  complex_matrix_value;
        dif          dif_value;
        difmatrix     difmatrix_value;
        name    name_value;
        Couple       couple_value;
        Triple       triple_value;
    } u;
} Value;

typedef struct Variable_tag {
    char	*name;
    Value	value;
    struct Variable_tag	*next;
} Variable;

typedef struct {
    Variable	*variable;
} LocalEnvironment;


typedef enum {
    SLAB_TTY_INPUT_MODE = 1,
    SLAB_FILE_INPUT_MODE
} SLAB_InputMode;


struct SLAB_Interpreter_tag {
    SLAB_InputMode	input_mode;
    FILE		*input_fp;
    MEM_Storage		global_storage;
    MEM_Storage		current_storage;
    Variable		*variable;
    FunctionDefinition	*function_list;
    jmp_buf		error_recovery_environment;
};

typedef struct SLAB_Interpreter_tag *SLAB_Interpreter;
SLAB_Interpreter SLAB_create_interpreter(void);
void SLAB_interpret(SLAB_Interpreter interpreter, FILE *fp,
		   SLAB_InputMode input_mode);
void SLAB_dispose_interpreter(SLAB_Interpreter interpreter);


#ifdef GLOBAL_VARIABLE_DEFINE
#define GLOBAL
#else
#define GLOBAL extern
#endif

GLOBAL SLAB_Interpreter clc_current_interpreter;

/* scanner.l */
void clc_initialize_readline_buffer(void);

/* create.c */
Expression
convert_value_to_expression(Value *v);
void clc_function_define(char *identifier, ParameterList *parameter_list,
			 Expression *expression_list);
ParameterList *clc_create_parameter(char *identifier);
ParameterList *clc_chain_parameter(ParameterList *list,
				   char *identifier);
Expression *clc_alloc_expression(ExpressionType type);
Expression *clc_create_expression_list(Expression *expression);
Expression *clc_chain_expression_list(Expression *list, Expression *add);
Expression *clc_create_assign_expression(char *variable,
					 Expression *operand);
void clc_save_data(char *dentifier,
					 char *data_file);
Expression *clc_create_binary_expression(ExpressionType operator,
					 Expression *left,
					 Expression *right);
Expression *clc_create_pow_expression(Expression *left, Expression *right);
Expression *clc_create_minus_expression(Expression *operand);
Expression *clc_create_dif_expression(Expression *operand, Expression *n, Expression *i);
Expression *clc_create_name_expression(char *name);
Expression *clc_create_norm_expression(Expression *operand);
Expression *clc_create_up_expression(Expression *operand);
Expression *clc_create_down_expression(Expression *operand);
Expression *clc_create_down_expression2(void);
Expression *clc_create_tic_expression(void);
Expression *clc_create_toc_expression(void);
Expression *clc_create_up_expression2(void);
Expression *clc_create_near_expression2(void);
Expression *clc_create_near_expression(Expression *operand);
Expression *clc_create_getbits_expression(Expression *operand);
Expression *clc_create_inv_expression(Expression *operand);
Expression *clc_create_transpose_expression(Expression *operand);
Expression *clc_create_eig_expression(Expression *operand);
Expression *clc_create_tri_expression(Expression *operand);
Expression *clc_create_veig_expression(Expression *operand);
Expression *clc_create_schur_expression(Expression *operand);
Expression *clc_create_lu_expression(Expression *operand);
Expression *clc_create_qr_expression(Expression *operand);
Expression *clc_create_chol_expression(Expression *operand);
Expression *clc_create_svd_expression(Expression *operand);
Expression *clc_create_vsin_expression(Expression *operand);
Expression *clc_create_sin_expression(Expression *operand);
Expression *clc_create_cos_expression(Expression *operand);
Expression *clc_create_vcos_expression(Expression *operand);
Expression *clc_create_log_expression(Expression *operand);
Expression *clc_create_vlog_expression(Expression *operand);
Expression *clc_create_log10_expression(Expression *operand);
Expression *clc_create_exp_expression(Expression *operand);
Expression *clc_create_vexp_expression(Expression *operand);
Expression *clc_create_fabs_expression(Expression *operand);
Expression *clc_create_tan_expression(Expression *operand);
Expression *clc_create_vtan_expression(Expression *operand);
Expression *clc_create_atan_expression(Expression *operand);
Expression *clc_create_acos_expression(Expression *operand);
Expression *clc_create_asin_expression(Expression *operand);
Expression *clc_create_cosh_expression(Expression *operand);
Expression *clc_create_sinh_expression(Expression *operand);
Expression *clc_create_tanh_expression(Expression *operand);
Expression *clc_create_sqrt_expression(Expression *operand);
Expression *clc_create_split_expression(Expression *operand);
Expression *clc_create_size_expression(Expression *operand);
Expression *clc_create_identifier_expression(char *identifier);
Expression *clc_create_couple_identifier_expression(char *identifier, char *name);
Expression *clc_create_print_expression(char *identifier);
Expression *clc_create_plot_expression(char *identifier, char *name_second);
Expression *clc_create_plot2_expression(char *identifier, char *name_second);
Expression *clc_create_plot3_expression(char *identifier, char *name_second, char *name_third);
Expression *clc_create_linpro_expression(char *identifier, char *name_second, char *name_third);
Expression *clc_create_who_expression(void);
Expression *clc_create_for_expression(Expression *first_condition,
				     Expression *second_condition,
				     Expression *third_condition,
                                     Expression *expression);
Expression *clc_create_if_expression(Expression *condition,
				     Expression *then_expression,
				     Expression *else_expression);
Expression *clc_create_if_expression2(Expression *condition,
				     Expression *then_expression);
Expression *clc_create_while_expression(Expression *condition,
					Expression *expression);
Expression *clc_create_do_while_expression(Expression *expression,
					Expression *condition);
Expression *clc_create_matrix_ones_expression(Expression *row,
					Expression *column);
Expression *clc_create_matrix_eye_expression(Expression *row,
					Expression *column);
Expression *clc_create_matrix_rand_expression(Expression *row,
					Expression *column);
Expression *clc_create_fft_expression(Expression *data,
					Expression *number);
Expression *clc_create_ifft_expression(Expression *data,
					Expression *number);
Expression *clc_create_matrix_zeros_expression(Expression *row,
					Expression *column);
Expression *clc_create_function_call_expression(char *func_name,
						Expression *expression);
Expression *clc_create_eval_expression(Expression *name,
						Expression *expression);
Expression *clc_create_con_expression(Expression *exp,
						Expression *next);
Expression *clc_create_max_expression(Expression *left,
						Expression *right);
Expression *clc_create_matrix_element_call_expression(char *mat_name,
					Expression *row, Expression *column);
Expression *clc_create_matrix_element_call_expression2(char *mat_name,
					Expression *row);
Expression *clc_create_linspace_expression(Expression *start,
					Expression *end, Expression *number);
Expression *clc_create_matrix_element_assign_expression(char *variable,
			Expression *row, Expression *column, Expression *operand);
Expression *clc_create_matrix_element_assign_expression2(char *variable,
			Expression *row, Expression *operand);
Expression *clc_create_matrix_expression(Expression *element);
Expression *clc_create_matrix(Expression *element);
Expression *clc_row_add_matrix(Expression *el, Expression *root);
Expression *clc_column_add_matrix(Expression *el, Expression *root);
Expression *clc_make_matrix_expression(long int column,long int row);
Expression *clc_init_matrix_expression(Expression *mat_el, long int column, long int row);
Expression *clc_is_matrix_expression(Expression *mat_el);
Expression *clc_create_matrix_element_expression(Expression *element);
Expression *clc_is_matrix_value_expression(LocalEnvironment *env, Expression *mat_el);
Expression * clc_init_matrix_value_expression(LocalEnvironment *env, Expression *mat_el, long int column, long int row, int matrix_type);
Expression *clc_make_matrix_value_expression(long int column, long int row, int matrix_type);
Expression *clc_create_interval_value_expression(Expression *lower, Expression *upper);
void clc_clear(void);
Expression *clc_create_couple_expression(char *name_first, char *name_second);
Expression *clc_create_triple_expression(char *name_first, char *name_second, char *name_third);
/* eval.c */
void  clc_eval_expression(Expression *expr);
void  eval_print_expression(LocalEnvironment *env, char *identifier);
void  eval_plot2_expression(LocalEnvironment *env, char *identifier, char *name_second);
void  eval_plot3_expression(LocalEnvironment *env, char *identifier, char *name_second, char *name_third);
void  eval_linpro_expression(LocalEnvironment *env, char *identifier, char *name_second, char *name_third);
Value eval_expression(LocalEnvironment *env,Expression *expr);
Value clc_eval_binary_expression(LocalEnvironment *env,
				 ExpressionType operator,
				 Expression *left, Expression *right);
Value clc_eval_norm_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_minus_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_up_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_down_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_near_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_getbits_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_inv_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_real_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_imag_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_upper_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_lower_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_transpose_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_eig_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_tri_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_schur_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_lu_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_qr_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_vsin_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_sin_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_sin(double r, double i);
double imaginary_part_of_complex_sin(double r, double i);
double real_part_of_complex_vsin(double r, double i);
double imaginary_part_of_complex_vsin(double r, double i);
Value clc_eval_cos_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_vcos_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_cos(double r, double i);
double imaginary_part_of_complex_cos(double r, double i);
double real_part_of_complex_vcos(double r, double i);
double imaginary_part_of_complex_vcos(double r, double i);
Value clc_eval_log_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_vlog_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_log(double r, double i);
double imaginary_part_of_complex_log(double r, double i);
double real_part_of_complex_vlog(double r, double i);
double imaginary_part_of_complex_vlog(double r, double i);
Value clc_eval_log10_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_exp_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_vexp_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_exp(double r, double i);
double imaginary_part_of_complex_exp(double r, double i);
double real_part_of_complex_vexp(double r, double i);
double imaginary_part_of_complex_vexp(double r, double i);
Value clc_eval_fabs_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_tan_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_vtan_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_tan(double r, double i);
double imaginary_part_of_complex_tan(double r, double i);
double *complex_vtan(double r, double i);
Value clc_eval_atan_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_acos_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_asin_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_cosh_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_sinh_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_tanh_expression(LocalEnvironment *env, Expression *operand);
Value clc_eval_sqrt_expression(LocalEnvironment *env, Expression *operand);
double real_part_of_complex_sqrt(double r, double i);
double imaginary_part_of_complex_sqrt(double r, double i);
Value clc_eval_matrix_expression(LocalEnvironment *env, Expression *matrix_expression,
                                  long int column, long int row);
Value clc_eval_matrix_value_expression(LocalEnvironment *env, double *mat,
                                  long int column, long int row);
Value clc_eval_complex_matrix_value_expression(LocalEnvironment *env, Complex *mat,
                                  long int column, long int row);
Value clc_eval_matrix_element_expression(LocalEnvironment *env, Expression *matrix_expression);
Value eval_interval_matrix_value_expression(matr lower, matr up);
Value eval_interval_expression(LocalEnvironment *env, Expression *lower, Expression *upper);
void clc_fprint_value(Value v, FILE *fp);
/* util.c */
void *clc_malloc(size_t size);
Value *clc_search_local_variable(LocalEnvironment *env,
				 char *identifier);
Value *clc_search_global_variable(char *identifier);
Value *clc_search_local_variable(LocalEnvironment *env,
				 char *identifier);
Value *clc_search_global_variable(char *identifier);
void clc_add_local_variable(LocalEnvironment *env,
			    char *identifier, Value *value);
void clc_add_global_variable(char *identifier, Value *value);
FunctionDefinition *clc_search_function(char *name);
void clc_reopen_current_storage(void);
Value eval_interval_value_expression(double lower,double upper);
/* error.c */
void clc_compile_error(CompileError id, char *fmt, ...);
void clc_runtime_error(RuntimeError id, char *fmt, ...);

/* round.c */
void up(void);
void down(void);
void near(void);

/* matrix.c */
double slb_matrix_norm(long int no_row, long int no_column, double *a);
void slb_matrix_multyply(double *a,double *b,double *c,long int m,long int n,long int k);
void slb_complex_matrix_multyply(Complex *a, Complex *b, Complex *c,long int m,long int n,long int k);
void slb_matrix_inverse(double *A, double *c, long int n);
void slb_triangular_matrix_inverse(double *A, double *c, char uplo, char diag, long int n);
void slb_complex_triangular_matrix_inverse(Complex *A, Complex *c, char uplo, char diag, long int n);
void slb_complex_matrix_inverse(Complex *A, Complex *c, long int n);
void slb_matrix_eigenvalue(double *A, double *w, long int n);
void slb_he_matrix_eigenvalue(Complex *A, double *w, long int n);
void slb_ger_matrix_eigenvalue(double *A, double *w_real, double *w_imag, double *vec, long int n);
void slb_gec_matrix_eigenvalue(Complex *A, Complex *w_complex, Complex *vr, long int n);
void slb_schur(double *A, double *vs, long int n);
void slb_complex_schur(Complex *A, Complex *vs, long int n);
void slb_qr(double *A, double *B, long int k, long int l);
void slb_chol(double *A, long int l);
void slb_complex_chol(Complex *A, long int l);
void slb_lu(double *A, double *B, long int k, long int l);
void slb_complex_lu(Complex *A, double *B, long int k, long int l);
void slb_svd(double *A, double *u, double *s, double *v,long int k, long int l);
void slb_complex_svd(Complex *A, Complex *u, double *s, Complex *v, long int k, long int l);
void slab_app_solve_matrix_equation(double *A, double *B, double *x,long int N,long int m);
void slab_app_solve_complex_matrix_equation(Complex *A, Complex *B, Complex *x,long int N, long int m);
void slab_solve_matrix_equation(double *A, double *B, double *x,long int N, long int m);
void slb_complex_qr(Complex *A, Complex *B, long int k, long int l);

void *clc_global_malloc(size_t size);

#endif /* PRIVATE_CALC_H_INCLUDED */
